@@ -4,7 +4,9 @@ from dataclasses import dataclass
 
 from mitmproxy import http
 from mitmproxy.connection import Connection
-from mitmproxy.proxy import commands, events, layer
+from mitmproxy.proxy import commands
+from mitmproxy.proxy import events
+from mitmproxy.proxy import layer
 from mitmproxy.proxy.context import Context
 
 StreamId = int
@@ -40,7 +42,9 @@ class ReceiveHttp(HttpCommand):
 
 def format_error(status_code: int, message: str) -> bytes:
     reason = http.status_codes.RESPONSES.get(status_code, "Unknown")
-    return textwrap.dedent(f"""
+    return (
+        textwrap.dedent(
+            f"""
     <html>
     <head>
         <title>{status_code} {reason}</title>
@@ -50,4 +54,8 @@ def format_error(status_code: int, message: str) -> bytes:
         <p>{html.escape(message)}</p>
     </body>
     </html>
-    """).strip().encode("utf8", "replace")
+    """
+        )
+        .strip()
+        .encode("utf8", "replace")
+    )
