@@ -1,7 +1,12 @@
 import collections
 import time
+from collections.abc import Sequence
 from enum import Enum
-from typing import ClassVar, DefaultDict, Dict, List, Optional, Sequence, Tuple, Type, Union
+from logging import DEBUG
+from logging import ERROR
+from typing import Any
+from typing import assert_never
+from typing import ClassVar
 
 import h2.config
 import h2.connection
@@ -12,19 +17,41 @@ import h2.settings
 import h2.stream
 import h2.utilities
 
-from mitmproxy import http, version
-from mitmproxy.connection import Connection
-from mitmproxy.net.http import status_codes, url
-from mitmproxy.utils import human
-from . import RequestData, RequestEndOfMessage, RequestHeaders, RequestProtocolError, ResponseData, \
-    ResponseEndOfMessage, ResponseHeaders, RequestTrailers, ResponseTrailers, ResponseProtocolError
-from ._base import HttpConnection, HttpEvent, ReceiveHttp, format_error
-from ._http_h2 import BufferedH2Connection, H2ConnectionLogger
-from ...commands import CloseConnection, Log, SendData
+from ...commands import CloseConnection
+from ...commands import Log
+from ...commands import RequestWakeup
+from ...commands import SendData
 from ...context import Context
-from ...events import ConnectionClosed, DataReceived, Event, Start
+from ...events import ConnectionClosed
+from ...events import DataReceived
+from ...events import Event
+from ...events import Start
+from ...events import Wakeup
 from ...layer import CommandGenerator
 from ...utils import expect
+from . import ErrorCode
+from . import RequestData
+from . import RequestEndOfMessage
+from . import RequestHeaders
+from . import RequestProtocolError
+from . import RequestTrailers
+from . import ResponseData
+from . import ResponseEndOfMessage
+from . import ResponseHeaders
+from . import ResponseProtocolError
+from . import ResponseTrailers
+from ._base import format_error
+from ._base import HttpConnection
+from ._base import HttpEvent
+from ._base import ReceiveHttp
+from ._http_h2 import BufferedH2Connection
+from ._http_h2 import H2ConnectionLogger
+from mitmproxy import http
+from mitmproxy import version
+from mitmproxy.connection import Connection
+from mitmproxy.net.http import status_codes
+from mitmproxy.net.http import url
+from mitmproxy.utils import human
 
 
 class StreamState(Enum):
@@ -37,7 +64,7 @@ CATCH_HYPER_H2_ERRORS = (ValueError, IndexError)
 
 class Http2Connection(HttpConnection):
     h2_conf: ClassVar[h2.config.H2Configuration]
-    h2_conf_defaults = dict(
+    h2_conf_defaults: dict[str, Any] = dict(
         header_encoding=False,
         validate_outbound_headers=False,
         # validate_inbound_headers is controlled by the validate_inbound_headers option.
@@ -45,20 +72,23 @@ class Http2Connection(HttpConnection):
         normalize_outbound_headers=False,
     )
     h2_conn: BufferedH2Connection
-    streams: Dict[int, StreamState]
+    streams: dict[int, StreamState]
     """keep track of all active stream ids to send protocol errors on teardown"""
 
-    ReceiveProtocolError: Type[Union[RequestProtocolError, ResponseProtocolError]]
-    ReceiveData: Type[Union[RequestData, ResponseData]]
-    ReceiveTrailers: Type[Union[RequestTrailers, ResponseTrailers]]
-    ReceiveEndOfMessage: Type[Union[RequestEndOfMessage, ResponseEndOfMessage]]
+    ReceiveProtocolError: type[RequestProtocolError | ResponseProtocolError]
+    ReceiveData: type[RequestData | ResponseData]
+    ReceiveTrailers: type[RequestTrailers | ResponseTrailers]
+    ReceiveEndOfMessage: type[RequestEndOfMessage | ResponseEndOfMessage]
 
     def __init__(self, context: Context, conn: Connection):
         super().__init__(context, conn)
         if self.debug:
-            self.h2_conf.logger = H2ConnectionLogger(f"{human.format_address(self.context.client.peername)}: "
-                                                     f"{self.__class__.__name__}")
-        self.h2_conf.validate_inbound_headers = self.context.options.validate_inbound_headers
+            self.h2_conf.logger = H2ConnectionLogger(
+                self.context.client.peername, self.__class__.__name__
+            )
+        self.h2_conf.validate_inbound_headers = (
+            self.context.options.validate_inbound_headers
+        )
         self.h2_conn = BufferedH2Connection(self.h2_conf)
         self.streams = {}
 
@@ -67,10 +97,9 @@ class Http2Connection(HttpConnection):
         stream = self.h2_conn.streams.get(stream_id, None)
         if (
             stream is not None
-            and
-            stream.state_machine.state is not h2.stream.StreamState.CLOSED
-            and
-            self.h2_conn.state_machine.state is not h2.connection.ConnectionState.CLOSED
+            and stream.state_machine.state is not h2.stream.StreamState.CLOSED
+            and self.h2_conn.state_machine.state
+            is not h2.connection.ConnectionState.CLOSED
         ):
             return False
         else:
@@ -81,12 +110,11 @@ class Http2Connection(HttpConnection):
         stream = self.h2_conn.streams.get(stream_id, None)
         if (
             stream is not None
-            and
-            stream.state_machine.state is not h2.stream.StreamState.HALF_CLOSED_LOCAL
-            and
-            stream.state_machine.state is not h2.stream.StreamState.CLOSED
-            and
-            self.h2_conn.state_machine.state is not h2.connection.ConnectionState.CLOSED
+            and stream.state_machine.state
+            is not h2.stream.StreamState.HALF_CLOSED_LOCAL
+            and stream.state_machine.state is not h2.stream.StreamState.CLOSED
+            and self.h2_conn.state_machine.state
+            is not h2.connection.ConnectionState.CLOSED
         ):
             return True
         else:
@@ -104,35 +132,59 @@ class Http2Connection(HttpConnection):
             elif isinstance(event, (RequestTrailers, ResponseTrailers)):
                 if self.is_open_for_us(event.stream_id):
                     trailers = [*event.trailers.fields]
-                    self.h2_conn.send_headers(event.stream_id, trailers, end_stream=True)
+                    self.h2_conn.send_trailers(event.stream_id, trailers)
             elif isinstance(event, (RequestEndOfMessage, ResponseEndOfMessage)):
                 if self.is_open_for_us(event.stream_id):
                     self.h2_conn.end_stream(event.stream_id)
             elif isinstance(event, (RequestProtocolError, ResponseProtocolError)):
                 if not self.is_closed(event.stream_id):
-                    code = {
-                        status_codes.CLIENT_CLOSED_REQUEST: h2.errors.ErrorCodes.CANCEL,
-                    }.get(event.code, h2.errors.ErrorCodes.INTERNAL_ERROR)
                     stream: h2.stream.H2Stream = self.h2_conn.streams[event.stream_id]
-                    send_error_message = (
+                    status = event.code.http_status_code()
+                    if (
                         isinstance(event, ResponseProtocolError)
                         and self.is_open_for_us(event.stream_id)
                         and not stream.state_machine.headers_sent
-                        and event.code != status_codes.NO_RESPONSE
-                    )
-                    if send_error_message:
-                        self.h2_conn.send_headers(event.stream_id, [
-                            (b":status", b"%d" % event.code),
-                            (b"server", version.MITMPROXY.encode()),
-                            (b"content-type", b"text/html"),
-                        ])
+                        and status is not None
+                    ):
+                        self.h2_conn.send_headers(
+                            event.stream_id,
+                            [
+                                (b":status", b"%d" % status),
+                                (b"server", version.MITMPROXY.encode()),
+                                (b"content-type", b"text/html"),
+                            ],
+                        )
                         self.h2_conn.send_data(
                             event.stream_id,
-                            format_error(event.code, event.message),
-                            end_stream=True
+                            format_error(status, event.message),
+                            end_stream=True,
                         )
                     else:
-                        self.h2_conn.reset_stream(event.stream_id, code)
+                        match event.code:
+                            case ErrorCode.CANCEL | ErrorCode.CLIENT_DISCONNECTED:
+                                error_code = h2.errors.ErrorCodes.CANCEL
+                            case ErrorCode.KILL:
+                                # XXX: Debateable whether this is the best error code.
+                                error_code = h2.errors.ErrorCodes.INTERNAL_ERROR
+                            case ErrorCode.HTTP_1_1_REQUIRED:
+                                error_code = h2.errors.ErrorCodes.HTTP_1_1_REQUIRED
+                            case ErrorCode.PASSTHROUGH_CLOSE:
+                                # FIXME: This probably shouldn't be a protocol error, but an EOM event.
+                                error_code = h2.errors.ErrorCodes.CANCEL
+                            case (
+                                ErrorCode.GENERIC_CLIENT_ERROR
+                                | ErrorCode.GENERIC_SERVER_ERROR
+                                | ErrorCode.REQUEST_TOO_LARGE
+                                | ErrorCode.RESPONSE_TOO_LARGE
+                                | ErrorCode.CONNECT_FAILED
+                                | ErrorCode.DESTINATION_UNKNOWN
+                                | ErrorCode.REQUEST_VALIDATION_FAILED
+                                | ErrorCode.RESPONSE_VALIDATION_FAILED
+                            ):
+                                error_code = h2.errors.ErrorCodes.INTERNAL_ERROR
+                            case other:  # pragma: no cover
+                                assert_never(other)
+                        self.h2_conn.reset_stream(event.stream_id, error_code.value)
             else:
                 raise AssertionError(f"Unexpected event: {event}")
             data_to_send = self.h2_conn.data_to_send()
@@ -147,16 +199,18 @@ class Http2Connection(HttpConnection):
                     # this should never raise a ValueError, but we triggered one while fuzzing:
                     # https://github.com/python-hyper/hyper-h2/issues/1231
                     # this stays here as defense-in-depth.
-                    raise h2.exceptions.ProtocolError(f"uncaught hyper-h2 error: {e}") from e
+                    raise h2.exceptions.ProtocolError(
+                        f"uncaught hyper-h2 error: {e}"
+                    ) from e
             except h2.exceptions.ProtocolError as e:
                 events = [e]
 
             for h2_event in events:
                 if self.debug:
-                    yield Log(f"{self.debug}[h2] {h2_event}", "debug")
+                    yield Log(f"{self.debug}[h2] {h2_event}", DEBUG)
                 if (yield from self.handle_h2_event(h2_event)):
                     if self.debug:
-                        yield Log(f"{self.debug}[h2] done", "debug")
+                        yield Log(f"{self.debug}[h2] done", DEBUG)
                     return
 
             data_to_send = self.h2_conn.data_to_send()
@@ -173,11 +227,17 @@ class Http2Connection(HttpConnection):
         if isinstance(event, h2.events.DataReceived):
             state = self.streams.get(event.stream_id, None)
             if state is StreamState.HEADERS_RECEIVED:
-                yield ReceiveHttp(self.ReceiveData(event.stream_id, event.data))
+                is_empty_eos_data_frame = event.stream_ended and not event.data
+                if not is_empty_eos_data_frame:
+                    yield ReceiveHttp(self.ReceiveData(event.stream_id, event.data))
             elif state is StreamState.EXPECTING_HEADERS:
-                yield from self.protocol_error(f"Received HTTP/2 data frame, expected headers.")
+                yield from self.protocol_error(
+                    f"Received HTTP/2 data frame, expected headers."
+                )
                 return True
-            self.h2_conn.acknowledge_received_data(event.flow_controlled_length, event.stream_id)
+            self.h2_conn.acknowledge_received_data(
+                event.flow_controlled_length, event.stream_id
+            )
         elif isinstance(event, h2.events.TrailersReceived):
             trailers = http.Headers(event.headers)
             yield ReceiveHttp(self.ReceiveTrailers(event.stream_id, trailers))
@@ -195,11 +255,20 @@ class Http2Connection(HttpConnection):
                     err_str = h2.errors.ErrorCodes(event.error_code).name
                 except ValueError:
                     err_str = str(event.error_code)
-                err_code = {
-                    h2.errors.ErrorCodes.CANCEL: status_codes.CLIENT_CLOSED_REQUEST,
-                }.get(event.error_code, self.ReceiveProtocolError.code)
-                yield ReceiveHttp(self.ReceiveProtocolError(event.stream_id, f"stream reset by client ({err_str})",
-                                                            code=err_code))
+                match event.error_code:
+                    case h2.errors.ErrorCodes.CANCEL:
+                        err_code = ErrorCode.CANCEL
+                    case h2.errors.ErrorCodes.HTTP_1_1_REQUIRED:
+                        err_code = ErrorCode.HTTP_1_1_REQUIRED
+                    case _:
+                        err_code = self.ReceiveProtocolError.code
+                yield ReceiveHttp(
+                    self.ReceiveProtocolError(
+                        event.stream_id,
+                        f"stream reset by client ({err_str})",
+                        code=err_code,
+                    )
+                )
                 self.streams.pop(event.stream_id)
             else:
                 pass  # We don't track priority frames which could be followed by a stream reset here.
@@ -226,11 +295,18 @@ class Http2Connection(HttpConnection):
         elif isinstance(event, h2.events.PingAckReceived):
             pass
         elif isinstance(event, h2.events.PushedStreamReceived):
-            yield Log("Received HTTP/2 push promise, even though we signalled no support.", "error")
+            yield Log(
+                "Received HTTP/2 push promise, even though we signalled no support.",
+                ERROR,
+            )
         elif isinstance(event, h2.events.UnknownFrameReceived):
             # https://http2.github.io/http2-spec/#rfc.section.4.1
             # Implementations MUST ignore and discard any frame that has a type that is unknown.
             yield Log(f"Ignoring unknown HTTP/2 frame type: {event.frame.type}")
+        elif isinstance(event, h2.events.AlternativeServiceAvailable):
+            yield Log(
+                "Received HTTP/2 Alt-Svc frame, which will not be forwarded.", DEBUG
+            )
         else:
             raise AssertionError(f"Unexpected event: {event!r}")
         return False
@@ -248,35 +324,85 @@ class Http2Connection(HttpConnection):
     def close_connection(self, msg: str) -> CommandGenerator[None]:
         yield CloseConnection(self.conn)
         for stream_id in self.streams:
-            yield ReceiveHttp(self.ReceiveProtocolError(stream_id, msg))
+            yield ReceiveHttp(
+                self.ReceiveProtocolError(
+                    stream_id, msg, self.ReceiveProtocolError.code
+                )
+            )
         self.streams.clear()
         self._handle_event = self.done  # type: ignore
 
-    @expect(DataReceived, HttpEvent, ConnectionClosed)
+    @expect(DataReceived, HttpEvent, ConnectionClosed, Wakeup)
     def done(self, _) -> CommandGenerator[None]:
         yield from ()
 
 
-def normalize_h1_headers(headers: List[Tuple[bytes, bytes]], is_client: bool) -> List[Tuple[bytes, bytes]]:
+def normalize_h1_headers(
+    headers: list[tuple[bytes, bytes]], is_client: bool
+) -> list[tuple[bytes, bytes]]:
     # HTTP/1 servers commonly send capitalized headers (Content-Length vs content-length),
     # which isn't valid HTTP/2. As such we normalize.
-    headers = h2.utilities.normalize_outbound_headers(
-        headers,
-        h2.utilities.HeaderValidationFlags(is_client, False, not is_client, False)
-    )
-    # make sure that this is not just an iterator but an iterable,
+    # Make sure that this is not just an iterator but an iterable,
     # otherwise hyper-h2 will silently drop headers.
-    headers = list(headers)
-    return headers
+    return list(
+        h2.utilities.normalize_outbound_headers(
+            headers,
+            h2.utilities.HeaderValidationFlags(is_client, False, not is_client, False),
+        )
+    )
 
 
-def normalize_h2_headers(headers: List[Tuple[bytes, bytes]]) -> CommandGenerator[None]:
+def normalize_h2_headers(headers: list[tuple[bytes, bytes]]) -> CommandGenerator[None]:
     for i in range(len(headers)):
         if not headers[i][0].islower():
-            yield Log(f"Lowercased {repr(headers[i][0]).lstrip('b')} header as uppercase is not allowed with HTTP/2.")
+            yield Log(
+                f"Lowercased {repr(headers[i][0]).lstrip('b')} header as uppercase is not allowed with HTTP/2."
+            )
             headers[i] = (headers[i][0].lower(), headers[i][1])
 
 
+def format_h2_request_headers(
+    context: Context,
+    event: RequestHeaders,
+) -> CommandGenerator[list[tuple[bytes, bytes]]]:
+    pseudo_headers = [
+        (b":method", event.request.data.method),
+        (b":scheme", event.request.data.scheme),
+        (b":path", event.request.data.path),
+    ]
+    if event.request.authority:
+        pseudo_headers.append((b":authority", event.request.data.authority))
+
+    if event.request.is_http2 or event.request.is_http3:
+        hdrs = list(event.request.headers.fields)
+        if context.options.normalize_outbound_headers:
+            yield from normalize_h2_headers(hdrs)
+    else:
+        headers = event.request.headers
+        if not event.request.authority and "host" in headers:
+            headers = headers.copy()
+            pseudo_headers.append((b":authority", headers.pop(b"host")))
+        hdrs = normalize_h1_headers(list(headers.fields), True)
+
+    return pseudo_headers + hdrs
+
+
+def format_h2_response_headers(
+    context: Context,
+    event: ResponseHeaders,
+) -> CommandGenerator[list[tuple[bytes, bytes]]]:
+    headers = [
+        (b":status", b"%d" % event.response.status_code),
+        *event.response.headers.fields,
+    ]
+    if event.response.is_http2 or event.response.is_http3:
+        if context.options.normalize_outbound_headers:
+            yield from normalize_h2_headers(headers)
+    else:
+        headers = normalize_h1_headers(headers, False)
+    return headers
+
+
 class Http2Server(Http2Connection):
     h2_conf = h2.config.H2Configuration(
         **Http2Connection.h2_conf_defaults,
@@ -294,19 +420,11 @@ class Http2Server(Http2Connection):
     def _handle_event(self, event: Event) -> CommandGenerator[None]:
         if isinstance(event, ResponseHeaders):
             if self.is_open_for_us(event.stream_id):
-                headers = [
-                    (b":status", b"%d" % event.response.status_code),
-                    *event.response.headers.fields
-                ]
-                if event.response.is_http2:
-                    if self.context.options.normalize_outbound_headers:
-                        yield from normalize_h2_headers(headers)
-                else:
-                    headers = normalize_h1_headers(headers, False)
-
                 self.h2_conn.send_headers(
                     event.stream_id,
-                    headers,
+                    headers=(
+                        yield from format_h2_response_headers(self.context, event)
+                    ),
                     end_stream=event.end_stream,
                 )
                 yield SendData(self.conn, self.h2_conn.data_to_send())
@@ -316,7 +434,15 @@ class Http2Server(Http2Connection):
     def handle_h2_event(self, event: h2.events.Event) -> CommandGenerator[bool]:
         if isinstance(event, h2.events.RequestReceived):
             try:
-                host, port, method, scheme, authority, path, headers = parse_h2_request_headers(event.headers)
+                (
+                    host,
+                    port,
+                    method,
+                    scheme,
+                    authority,
+                    path,
+                    headers,
+                ) = parse_h2_request_headers(event.headers)
             except ValueError as e:
                 yield from self.protocol_error(f"Invalid HTTP/2 request headers: {e}")
                 return True
@@ -335,7 +461,11 @@ class Http2Server(Http2Connection):
                 timestamp_end=None,
             )
             self.streams[event.stream_id] = StreamState.HEADERS_RECEIVED
-            yield ReceiveHttp(RequestHeaders(event.stream_id, request, end_stream=bool(event.stream_ended)))
+            yield ReceiveHttp(
+                RequestHeaders(
+                    event.stream_id, request, end_stream=bool(event.stream_ended)
+                )
+            )
             return False
         else:
             return (yield from super().handle_h2_event(event))
@@ -352,12 +482,14 @@ class Http2Client(Http2Connection):
     ReceiveTrailers = ResponseTrailers
     ReceiveEndOfMessage = ResponseEndOfMessage
 
-    our_stream_id: Dict[int, int]
-    their_stream_id: Dict[int, int]
-    stream_queue: DefaultDict[int, List[Event]]
+    our_stream_id: dict[int, int]
+    their_stream_id: dict[int, int]
+    stream_queue: collections.defaultdict[int, list[Event]]
     """Queue of streams that we haven't sent yet because we have reached MAX_CONCURRENT_STREAMS"""
-    provisional_max_concurrency: Optional[int] = 10
+    provisional_max_concurrency: int | None = 10
     """A provisional currency limit before we get the server's first settings frame."""
+    last_activity: float
+    """Timestamp of when we've last seen network activity on this connection."""
 
     def __init__(self, context: Context):
         super().__init__(context, context.server)
@@ -377,9 +509,9 @@ class Http2Client(Http2Connection):
         if isinstance(event, HttpEvent):
             ours = self.our_stream_id.get(event.stream_id, None)
             if ours is None:
-                no_free_streams = (
-                    self.h2_conn.open_outbound_streams >=
-                    (self.provisional_max_concurrency or self.h2_conn.remote_settings.max_concurrent_streams)
+                no_free_streams = self.h2_conn.open_outbound_streams >= (
+                    self.provisional_max_concurrency
+                    or self.h2_conn.remote_settings.max_concurrent_streams
                 )
                 if no_free_streams:
                     self.stream_queue[event.stream_id].append(event)
@@ -394,11 +526,9 @@ class Http2Client(Http2Connection):
                 cmd.event.stream_id = self.their_stream_id[cmd.event.stream_id]
             yield cmd
 
-        can_resume_queue = (
-            self.stream_queue and
-            self.h2_conn.open_outbound_streams < (
-                self.provisional_max_concurrency or self.h2_conn.remote_settings.max_concurrent_streams
-            )
+        can_resume_queue = self.stream_queue and self.h2_conn.open_outbound_streams < (
+            self.provisional_max_concurrency
+            or self.h2_conn.remote_settings.max_concurrent_streams
         )
         if can_resume_queue:
             # popitem would be LIFO, but we want FIFO.
@@ -407,29 +537,39 @@ class Http2Client(Http2Connection):
                 yield from self._handle_event(event)
 
     def _handle_event2(self, event: Event) -> CommandGenerator[None]:
-        if isinstance(event, RequestHeaders):
-            pseudo_headers = [
-                (b':method', event.request.data.method),
-                (b':scheme', event.request.data.scheme),
-                (b':path', event.request.data.path),
-            ]
-            if event.request.authority:
-                pseudo_headers.append((b":authority", event.request.data.authority))
-
-            if event.request.is_http2:
-                hdrs = list(event.request.headers.fields)
-                if self.context.options.normalize_outbound_headers:
-                    yield from normalize_h2_headers(hdrs)
-            else:
-                headers = event.request.headers
-                if not event.request.authority and "host" in headers:
-                    headers = headers.copy()
-                    pseudo_headers.append((b":authority", headers.pop(b"host")))
-                hdrs = normalize_h1_headers(list(headers.fields), True)
+        if isinstance(event, Wakeup):
+            send_ping_now = (
+                # add one second to avoid unnecessary roundtrip, we don't need to be super correct here.
+                time.time() - self.last_activity + 1
+                > self.context.options.http2_ping_keepalive
+            )
+            if send_ping_now:
+                # PING frames MUST contain 8 octets of opaque data in the payload.
+                # A sender can include any value it chooses and use those octets in any fashion.
+                self.last_activity = time.time()
+                self.h2_conn.ping(b"0" * 8)
+                data = self.h2_conn.data_to_send()
+                if data is not None:
+                    yield Log(
+                        f"Send HTTP/2 keep-alive PING to {human.format_address(self.conn.peername)}",
+                        DEBUG,
+                    )
+                    yield SendData(self.conn, data)
+            time_until_next_ping = self.context.options.http2_ping_keepalive - (
+                time.time() - self.last_activity
+            )
+            yield RequestWakeup(time_until_next_ping)
+            return
 
+        self.last_activity = time.time()
+        if isinstance(event, Start):
+            if self.context.options.http2_ping_keepalive > 0:
+                yield RequestWakeup(self.context.options.http2_ping_keepalive)
+            yield from super()._handle_event(event)
+        elif isinstance(event, RequestHeaders):
             self.h2_conn.send_headers(
                 event.stream_id,
-                pseudo_headers + hdrs,
+                headers=(yield from format_h2_request_headers(self.context, event)),
                 end_stream=event.end_stream,
             )
             self.streams[event.stream_id] = StreamState.EXPECTING_HEADERS
@@ -439,7 +579,10 @@ class Http2Client(Http2Connection):
 
     def handle_h2_event(self, event: h2.events.Event) -> CommandGenerator[bool]:
         if isinstance(event, h2.events.ResponseReceived):
-            if self.streams.get(event.stream_id, None) is not StreamState.EXPECTING_HEADERS:
+            if (
+                self.streams.get(event.stream_id, None)
+                is not StreamState.EXPECTING_HEADERS
+            ):
                 yield from self.protocol_error(f"Received unexpected HTTP/2 response.")
                 return True
 
@@ -460,10 +603,30 @@ class Http2Client(Http2Connection):
                 timestamp_end=None,
             )
             self.streams[event.stream_id] = StreamState.HEADERS_RECEIVED
-            yield ReceiveHttp(ResponseHeaders(event.stream_id, response, bool(event.stream_ended)))
+            yield ReceiveHttp(
+                ResponseHeaders(event.stream_id, response, bool(event.stream_ended))
+            )
+            return False
+        elif isinstance(event, h2.events.InformationalResponseReceived):
+            # We violate the spec here ("A proxy MUST forward 1xx responses", RFC 7231),
+            # but that's probably fine:
+            # - 100 Continue is sent by mitmproxy to clients (irrespective of what the server does).
+            # - 101 Switching Protocols is not allowed for HTTP/2.
+            # - 102 Processing is WebDAV only and also ignorable.
+            # - 103 Early Hints is not mission-critical.
+            headers = http.Headers(event.headers)
+            status: str | int = "<unknown status>"
+            try:
+                status = int(headers[":status"])
+                reason = status_codes.RESPONSES.get(status, "")
+            except (KeyError, ValueError):
+                reason = ""
+            yield Log(f"Swallowing HTTP/2 informational response: {status} {reason}")
             return False
         elif isinstance(event, h2.events.RequestReceived):
-            yield from self.protocol_error(f"HTTP/2 protocol error: received request from server")
+            yield from self.protocol_error(
+                f"HTTP/2 protocol error: received request from server"
+            )
             return True
         elif isinstance(event, h2.events.RemoteSettingsChanged):
             # We have received at least one settings from now,
@@ -474,10 +637,12 @@ class Http2Client(Http2Connection):
             return (yield from super().handle_h2_event(event))
 
 
-def split_pseudo_headers(h2_headers: Sequence[Tuple[bytes, bytes]]) -> Tuple[Dict[bytes, bytes], http.Headers]:
-    pseudo_headers: Dict[bytes, bytes] = {}
+def split_pseudo_headers(
+    h2_headers: Sequence[tuple[bytes, bytes]],
+) -> tuple[dict[bytes, bytes], http.Headers]:
+    pseudo_headers: dict[bytes, bytes] = {}
     i = 0
-    for (header, value) in h2_headers:
+    for header, value in h2_headers:
         if header.startswith(b":"):
             if header in pseudo_headers:
                 raise ValueError(f"Duplicate HTTP/2 pseudo header: {header!r}")
@@ -493,14 +658,16 @@ def split_pseudo_headers(h2_headers: Sequence[Tuple[bytes, bytes]]) -> Tuple[Dic
 
 
 def parse_h2_request_headers(
-    h2_headers: Sequence[Tuple[bytes, bytes]]
-) -> Tuple[str, int, bytes, bytes, bytes, bytes, http.Headers]:
+    h2_headers: Sequence[tuple[bytes, bytes]],
+) -> tuple[str, int, bytes, bytes, bytes, bytes, http.Headers]:
     """Split HTTP/2 pseudo-headers from the actual headers and parse them."""
     pseudo_headers, headers = split_pseudo_headers(h2_headers)
 
     try:
         method: bytes = pseudo_headers.pop(b":method")
-        scheme: bytes = pseudo_headers.pop(b":scheme")  # this raises for HTTP/2 CONNECT requests
+        scheme: bytes = pseudo_headers.pop(
+            b":scheme"
+        )  # this raises for HTTP/2 CONNECT requests
         path: bytes = pseudo_headers.pop(b":path")
         authority: bytes = pseudo_headers.pop(b":authority", b"")
     except KeyError as e:
@@ -512,7 +679,7 @@ def parse_h2_request_headers(
     if authority:
         host, port = url.parse_authority(authority, check=True)
         if port is None:
-            port = 80 if scheme == b'http' else 443
+            port = 80 if scheme == b"http" else 443
     else:
         host = ""
         port = 0
@@ -520,7 +687,9 @@ def parse_h2_request_headers(
     return host, port, method, scheme, authority, path, headers
 
 
-def parse_h2_response_headers(h2_headers: Sequence[Tuple[bytes, bytes]]) -> Tuple[int, http.Headers]:
+def parse_h2_response_headers(
+    h2_headers: Sequence[tuple[bytes, bytes]],
+) -> tuple[int, http.Headers]:
     """Split HTTP/2 pseudo-headers from the actual headers and parse them."""
     pseudo_headers, headers = split_pseudo_headers(h2_headers)
 
@@ -536,6 +705,10 @@ def parse_h2_response_headers(h2_headers: Sequence[Tuple[bytes, bytes]]) -> Tupl
 
 
 __all__ = [
+    "format_h2_request_headers",
+    "format_h2_response_headers",
+    "parse_h2_request_headers",
+    "parse_h2_response_headers",
     "Http2Client",
     "Http2Server",
 ]
